const prpl = require('prpl-server');
const express = require('express');
const helmet = require('helmet');
const app = express();

const bodyParser = require('body-parser');
const polyConfigFile = require("./build/polymer.json");
const port = 8081;

// Helmet sets various HTTP headers to enhance security
app.use(helmet({
  noSniff: false  // Disabled because with it, CORS rejects JSON responses
}));

// Use Node.js body parsing middleware
app.use(bodyParser.json());
app.use(bodyParser.urlencoded({
    extended: true,
}));

// Enable CORS
app.use((req, res, next) => {
  res.header("Access-Control-Allow-Origin", "*");
  res.header("Access-Control-Allow-Headers", "Origin, Content-Type, Accept, Keep-Alive, User-Agent");
  res.header("Access-Control-Allow-Methods", "GET, POST, PUT, DELETE, HEAD");
  next();
});

// App routes, see routes directory
require("./routes")(app);

// PRPL-server serves the app from a build
app.get('/*', prpl.makeHandler('server/build', polyConfigFile));

// App listens on port
app.listen(port, (error) => {
  if (error) return console.log('Error: ' + error);
  console.log('Express + prpl-server app listening on port ' + port);
});
