import * as express from 'express';
/**
 * A default set of user agent patterns for bots/crawlers that do not perform
 * well with pages that require JavaScript.
 */
export declare const botUserAgents: string[];
/**
 * Options for makeMiddleware.
 */
export interface Options {
    /**
     * Base URL of the Rendertron proxy service. Required.
     */
    proxyUrl: string;
    /**
     * Regular expression to match user agent to proxy. Defaults to a set of bots
     * that do not perform well with pages that require JavaScript.
     */
    userAgentPattern?: RegExp;
    /**
     * Regular expression used to exclude request URL paths. Defaults to a set of
     * typical static asset file extensions.
     */
    excludeUrlPattern?: RegExp;
    /**
     * Force web components polyfills to be loaded and enabled. Defaults to false.
     */
    injectShadyDom?: boolean;
    /**
     * Millisecond timeout for proxy requests. Defaults to 11000 milliseconds.
     */
    timeout?: number;
}
/**
 * Create a new Express middleware function that proxies requests to a
 * Rendertron bot rendering service.
 */
export declare function makeMiddleware(options: Options): express.Handler;
