const inbox = require('express').Router();
const pool = require('../../database.js');

var admin = require('firebase-admin');
admin.initializeApp({
  credential: admin.credential.applicationDefault(),
  databaseURL: 'https://bachelorprosjekt-spillerstatus.firebaseio.com'
});

// POST inbox - Add a new message to an injury/ inbox
inbox.post('/', async (request, response) => {
  let conn;
  try {
    conn = await pool.getConnection();
    const content = request.body.message;
    let toUser = request.body.to;
    const fromUser = request.body.from;
    const type = request.body.type;
    let injury = request.body.injury;
    let sql = '';
    const data = [
      fromUser,
      content
    ];
    let sqlReceivers = '';
    if(injury !== 'undefined'){
      data.push(injury);
      toUser = 0;
      sql = `CALL add_new_message_injury(?, ?, ?)`;
      // get all users tokens of people who is interested in this message being sent.

      sqlReceivers = `SELECT t.token FROM Token AS t
  						            INNER JOIN HealthProfessional AS hp ON t.user_id=hp.health_professional_id
                          INNER JOIN Contract AS c ON hp.organization_id=c.health_organization_id
                          INNER JOIN Athlete AS a ON c.team_id=a.team_id
                          INNER JOIN Injury AS i ON a.athlete_id=i.athlete_id
                          WHERE i.injury_id=?
                          UNION
                          SELECT t.token FROM Token AS t
                          INNER JOIN Injury AS i ON t.user_id=i.athlete_id
                          WHERE i.injury_id=?`;
    } else{
      data.push(toUser);
      injury = 0;
      sql = `CALL add_new_message_athlete(?, ?, ?)`;
      // get all users tokens of people who is interested in this message being sent.
      sqlReceivers = `SELECT t.token FROM Token AS t
  						            INNER JOIN HealthProfessional AS hp ON t.user_id=hp.health_professional_id
                          INNER JOIN Contract AS c ON hp.organization_id=c.health_organization_id
                          INNER JOIN Athlete AS a ON c.team_id=a.team_id
                          WHERE a.athlete_id=?
                          UNION
                          SELECT t.token FROM Token AS t
                          INNER JOIN Athlete AS a ON t.user_id=a.athlete_id
                          WHERE a.athlete_id=?`;
    }

    const row = await conn.query(sql, data);
    const fromName = row[0][0].first_name + ' ' + row[0][0].last_name;

    const dataReceivers = injury !== 'undefined' ? [ injury, injury ] : [ toUser, toUser ];
    const messageReceivers = await conn.query(sqlReceivers, dataReceivers);
    delete messageReceivers.meta;
    var d = new Date();
    const date = d.getDate() + '/' + d.getMonth() + '-' + d.getFullYear();

    notificationTitle = type !== 'undefined' ? 'Ny skade registrert' : 'Ny melding';
    if(messageReceivers.length > 0){
      injuryInboxId = String(injury);
      athleteInboxId = String(toUser);
      messages = [];
      for(let i = 0; i < messageReceivers.length; i++){
        const message = {
          notification: {
            title: notificationTitle
          },
          data: {
            fromName: fromName,
            fromUser: String(fromUser),
            id: String(row[0][0]['message_id']),
            injury: injuryInboxId,
            athlete: athleteInboxId,
            content: content,
            dateOfMessage: date
          },
          token: messageReceivers[i].token
        };
        messages.push(message);
      }
      admin.messaging().sendAll(messages)
        .then((response) => {
          console.log(response);
          console.log(response.successCount + ' messages were sent successfully');
        });
    } else {
      console.log('No tokens to send push notification to');
    }

    response.json({"status":"OK", "message": {id: row.insertId, injury: injury, athlete: toUser, from: fromName, content: content, date: date}});

  } catch (err) {
     response.json({"status":"ERR"});
    if (conn) return conn.end();
  }
});


/**
 * GET all messages in an inbox // TODO get other inboxes, not just with one athlete/ injury. Send healthprofessionalId instead of injury /inbox id.
 * Get all athlete the healthprofessional the id belongs to has a contract with.
 * Returns three objects: athletes, injuries, messages.
 * Athletes = athlete-objects (athlete-id) with info of the athlete and an object of injury-ids of each injuries an athlete has.
 * Injuries = injury-objects (injury-id) with info of the injury and an object of message-ids of each message about one injury.
 * Messages = message-objects (message-id) that contains its id, the full name of who the message was sent from and the content of the message.
 */
inbox.get('/:id', async (request, response) => {
  let conn;
  try {
    conn = await pool.getConnection();
    const userTypeData = [ request.params.id ]; // TODO add id for athletes the healthprofessional has a contract with
    const userTypeSql = 'SELECT user_type FROM User WHERE user_id=?';
    const userType = await conn.query(userTypeSql, userTypeData);

    const myAthletes = {};
    const myInjuries = {};
    const myMessages = {};
    const myUnreadMessages = {};
    let athletes;

    const unreadSql = 'SELECT message_id FROM UserUnreadMessage WHERE user_id=?';
    const unreadData = [ request.params.id ];
    const unreadRows = await conn.query(unreadSql, unreadData);
    delete unreadRows.meta;
    for(let i = 0; i < unreadRows.length; i++){
      myUnreadMessages[unreadRows[i].message_id] = unreadRows[i].message_id;
    }

    if(userType[0].user_type == 3){ // Get all athletes this healthprofessional has a contract with.
      const getAthletesSql = `SELECT u.user_id, u.first_name, u.last_name, cd.email
                              FROM User AS u
                              INNER JOIN ContactDetails AS cd ON u.user_id=cd.user_id
                              INNER JOIN Athlete AS a ON u.user_id=a.athlete_id
                              INNER JOIN Team AS t ON a.team_id=t.team_id
                              INNER JOIN Contract AS c ON t.team_id=c.team_id
                              INNER JOIN HealthProfessional AS hp ON c.health_organization_id=hp.organization_id
                              WHERE hp.health_professional_id=?`;

      const athleteRows = await conn.query(getAthletesSql, [request.params.id]);
      athletes = athleteRows;
      delete athletes.meta;

    } else if(userType[0].user_type == 1) { // Get all of the athletes injuries and messages about them.
      athletes = [{'user_id': request.params.id}];

    } else {
      // This is for when the user is a trainer. The trainer will not have an inbox in this project.
    }

    for(i = 0; i < athletes.length; i++){
      const athleteId = athletes[i].user_id;
      myAthletes[athleteId] = athletes[i];
      const injuryData = [ athleteId ];
      const injuriesSql = 'SELECT injury_id, title FROM Injury WHERE athlete_id=?';
      const injuryRows = await conn.query(injuriesSql, injuryData);
      const injuries = injuryRows;
      delete injuries.meta;

      myAthletes[athleteId]['messages'] = {};
      myAthletes[athleteId]['messages']['0'] = 0;
      // get all messages in athletes main thread
      const mainMessageData = [ athleteId ];
      const mainMessagesSql = 'SELECT message_id, user_id_from, user_id_to, inbox_injury_id, message, DATE_FORMAT(dateOfMessage, "%e/%m-%Y") as date FROM Message WHERE user_id_to=?';
      const mainMessageRows = await conn.query(mainMessagesSql, mainMessageData);
      const mainMessages = mainMessageRows;
      delete mainMessages.meta;

      for(k = 0; k < mainMessages.length; k++){
        const mainMessageId = mainMessages[k].message_id;
        myAthletes[athleteId]['messages'][mainMessageId] = mainMessageId;
        // get full name of the person who sent the message
        const userSql = 'SELECT first_name, last_name FROM User WHERE user_id=?';
        const userData = [mainMessages[k].user_id_from];
        const userRow = await conn.query(userSql, userData);
        const fullName = userRow[0].first_name + ' ' + userRow[0].last_name;
        myMessages[mainMessageId] = {"id": mainMessages[k].message_id, "from": fullName, "injury": mainMessages[k].inbox_injury_id, "athlete": mainMessages[k].user_id_to, "content": mainMessages[k].message, "date": mainMessages[k].date};
      }

      //if(injuries.length > 0){
        myAthletes[athleteId]['injuries'] = {};
	      myAthletes[athleteId]['injuries']['0'] = 0;
        for(j = 0; j < injuries.length; j++){
          const injuryId = injuries[j].injury_id;
          myAthletes[athleteId]['injuries'][injuryId] = injuryId;
          myInjuries[injuryId] = injuries[j];

          // get all messages about one injury
          const messageData = [ injuryId ];
          const messagesSql = 'SELECT message_id, user_id_from, user_id_to, inbox_injury_id, message, DATE_FORMAT(dateOfMessage, "%e/%m-%Y") as date FROM Message WHERE inbox_injury_id=?';
          const messageRows = await conn.query(messagesSql, messageData);
          const messages = messageRows;
          delete messages.meta;
          //if( messages.length > 0) {
	          myInjuries[injuryId]['messages'] = {};
            myInjuries[injuryId]['messages']['0'] = 0;

            for(k = 0; k < messages.length; k++){
              const messageId = messages[k].message_id;
              myInjuries[injuryId]['messages'][messageId] = messageId;
              // get full name of the person who sent the message
              const userSql = 'SELECT first_name, last_name FROM User WHERE user_id=?';
              const userData = [messages[k].user_id_from];
              const userRow = await conn.query(userSql, userData);
              const fullName = userRow[0].first_name + ' ' + userRow[0].last_name;
              myMessages[messageId] = {"id": messages[k].message_id, "from": fullName, "injury": messages[k].inbox_injury_id, "athlete": messages[k].user_id_to, "content": messages[k].message, "date": messages[k].date};
            }
          //}
        } // end for loop all injuries
      //}
    } // end for loop all athletes

    if(userType[0].user_type == 3){
      response.json({"status":"OK", "athletes": myAthletes, "injuries": myInjuries, "messages": myMessages, "newMessages": myUnreadMessages});
    } else if(userType[0].user_type == 1){  // TODO send one obj of the athlete.
      response.json({"status":"OK", "athletes": myAthletes, "injuries": myInjuries, "messages": myMessages, "newMessages": myUnreadMessages});
    } else {
      response.json({"status":"ERR", "error":"Ingen tilgang for trenere ennå"});
    }

  } catch (err){
    response.json({"status":"ERR", "error":err});
  } finally {
    if (conn) return conn.end();
  }

});

// Read a message
inbox.put('/', async (request, response) => {
  let conn;
  try {
    conn = await pool.getConnection();
    response.set("Content-Type", "application/json");
    const typeOfInbox = request.body.type;
    const messagesData = [ request.body.inbox ];
    let messageSql = '';
    if(typeOfInbox == "injury"){
      messageSql = 'SELECT message_id FROM Message WHERE inbox_injury_id=?';
    } else {
      messageSql = 'SELECT message_id FROM Message WHERE user_id_to=?';
    }
    const messages = await conn.query(messageSql, messagesData);
    delete messages.meta;
    let deleteData = [];
    for(let i = 0; i < messages.length; i++){
      deleteData.push([request.body.id, messages[i].message_id]);
    }


    let deleteSql = 'DELETE FROM UserUnreadMessage WHERE user_id=? AND message_id=?';
    for(let i = 0; i < deleteData.length; i++){
      const rows = await conn.query(deleteSql, deleteData[i]);
    }

    response.json({"status":"OK"});

  } catch (err) {
    response.json({"status":"ERR","error":err});
  } finally {
    if (conn) return conn.end();
  }
});

module.exports = inbox;
