import { LitElement, html, css } from  'lit-element';
import { SharedStyles } from './shared-styles.js';
import { connect } from 'pwa-helpers/connect-mixin.js';
import { store } from '../store.js';

import '@vaadin/vaadin-tabs';
import '@vaadin/vaadin-icons';
import '@vaadin/vaadin-button';
import '@vaadin/vaadin-custom-field';
import '@vaadin/vaadin-time-picker';
import '@vaadin/vaadin-date-picker';
import '@vaadin/vaadin-checkbox';
import '@vaadin/vaadin-notification';
import '@vaadin/vaadin-dialog';
import '@polymer/iron-pages';
import { GoogleCharts } from 'google-charts';


class ViewAthlete extends connect(store)(LitElement) {
  static get properties() {
    return {
      athlete: { type: Object },
      message: { type: String },
      messageTemp: { type: String},    
      autoMessageText: { type: String }, 
      ironPage: { type: Number},
      page: { type: Number },
      date: { type: String },
      time: { type: Number },
      _fullSocialNum: { type: String },
      todayDate: { type: String}
    }
  }

  static get styles() {
    return [
      SharedStyles,
      css`
        :host {
          display: block;
        }
        .name {
          text-align: center;
          display: block;
          font-size: 2em;
          -webkit-margin-before: 0.67em;
          -webkit-margin-after: 0.67em;
          -webkit-margin-start: 0px;
          -webkit-margin-end: 0px;
          font-weight: bold;
        }
        vaadin-tabs {
          box-shadow: none; 
        }
        #back {
          width: 30px;
          height: 30px;
        }
        #p1 {
          width: 30px;
          height: 30px;
        }
        #p2 {
          width: 30px;
          height: 30px;
        }
        #p3 {
          width: 30px;
          height: 30px;
        }
        #p4 {
          width: 30px;
          height: 30px;
        }
        textarea {
          margin-top: 10px;       
          width: 292px;
          height: 150px;
          -moz-border-bottom-colors: none;
          -moz-border-left-colors: none;
          -moz-border-right-colors: none;
          -moz-border-top-colors: none;
          background: none repeat scroll 0 0 rgba(0, 0, 0, 0.07);
          border-color: -moz-use-text-color #FFFFFF #FFFFFF -moz-use-text-color;
          border-image: none;
          border-radius: 6px 6px 6px 6px;
          border-style: none solid solid none;
          border-width: medium 1px 1px medium;
          box-shadow: 0 1px 2px rgba(0, 0, 0, 0.12) inset;
          color: #555555;
          font-family: "Helvetica Neue",Helvetica,Arial,sans-serif;
          font-size: 1em;
          line-height: 1.4em;
          padding: 5px 8px;
          transition: background-color 0.2s ease 0s;  
          resize: none
        }
        textarea:focus {
          background: none repeat scroll 0 0 #FFFFFF;
          outline-width: 0;
        }
        .text-area {
          text-align: center;        
        }        
        .button-group {
          display: flex;
          align-items: center;
          justify-content: center;          
        }
        .button-group vaadin-button {
          margin: 5px;
        }
        .date-time-group {
          display: flex;
          align-items: center;
          justify-content: center;    
        }
        .checkbox {
          text-align: center;
        }
        .checkbox p {
          font-size: var(--lumo-font-size-s);
          font-weight: 500;
          font-color: var(--lumo-secondary-text-color);
          position: relative;
          bottom: 2px;
        }
        #send-button {
          cursor: pointer;
        }
        #delete-button {
          cursor: pointer;
        }
      `
    ]
  }

  constructor() {
    super();
    this.athlete = {};
    this.message = '';
    this.messageTemp = '';
    this.ironPage = '1';
    this.page = '1';
    this.date = '';
    this.time = '';
    this._fullSocialNum = ''; 
    this.todayDate = '';
  }

  render() {
    this.autoMessageText = 'Hei ' + this.athlete.first_name + '.\nDu har time ' + this.date + ' kl. ' + this.time  + '.\n\nMed vennlig hilsen \nTerje Løkken,\nGjøvik Idrettsmedisinske Team';
    
    return html`
      <!-- Each 'iron-page' needs to be inside a <div>, preferably -->    
      <iron-pages selected="${this.ironPage}">
          
        <div>Tilbake</div>
          
        <div>
          <p class="name">${this.athlete.first_name} ${this.athlete.last_name}</p>
          <p>${this.athlete.team}</p>
          ${this.athlete.position ? html`<p>Posisjon: ${this.athlete.position}</p>` : html ``}
          ${this.athlete.shirt_no ? html`<p>Draktnummer: ${this.athlete.shirt_no}</p>` : html ``}
          <p>${this.athlete.email}</p>
          <p>Telefon: ${this.athlete.phone_num}</p>
          <p>Fødselsdato: ${this.athlete.birthdate}</p>
          <p>Personnummer: ${this._fullSocialNum}</p>
        </div>
        
        <div id="chart" style="width:400; height:300"></div>   
        
        <div>Rediger</div>
        
        <div>
          <div class="text-area">
            <textarea id="text-area" cols=30 rows="4" value="${this.message}" @keyup="${this.updateMessage}" placeholder="Skriv melding"></textarea>
            <!-- <textarea></textarea> must be on same line and no blanks between or else placeholder won't show  -->
          </div>  
          <div class="date-time-group">
            <vaadin-custom-field label="Tidspunkt for time">
              <vaadin-date-picker id="date-picker" min="${this.todayDate}" style="width: 153px;" 
              @change="${this.datePicked}" @click="${this.dateValue}"></vaadin-date-picker>
              <!-- The step property specified in seconds affects the resolution of the time picker and visibility of the 
              dropdown. If step is less than 15 mins dropdown is hidden. Here: set to 15 min intervall (900 sec)-->
              <vaadin-time-picker id="time-picker" step="900" style="width: 153px;" @click="${this.timePicked}"></vaadin-time-picker>            
            </vaadin-custom-field>
          </div>
          <div class="checkbox">
            <vaadin-checkbox id="checkbox" @change="${this.autoMessageGen}"><p>Autogenerer timebestillingsbeskjed</p></vaadin-checkbox>
          </div>
          <div class="button-group">
            <vaadin-button id="send-button" theme="primary small">Send</vaadin-button>
            <vaadin-button id="delete-button" theme="secondary small" @click="${this.deleteData}">Slett</vaadin-button>
          </div>
          <vaadin-dialog id="dialog" no-close-on-esc no-close-on-outside-click></vaadin-dialog>
          <vaadin-notification id="notification" duration="4000" position="bottom-center"></vaadin-notification>
        </div>          
        
      </iron-pages>  
      
      <div class="tabs">
        <vaadin-tabs theme="centered" selected="${this.page}">  
        
          <vaadin-tab @click="${this.back}">
              <iron-icon id="back" icon="vaadin:arrow-backward"></iron-icon>
              <span><!-- Put tex here to display next to icon --></span>
          </vaadin-tab>
    
          <vaadin-tab id="1" @click="${this.changeIronPage}">
            <iron-icon id="p1" icon="vaadin:user"></iron-icon>
            <span></span>
          </vaadin-tab>
          
          <vaadin-tab id="2" @click="${this.changeIronPage}">
            <iron-icon id="p2" icon="vaadin:chart"></iron-icon>
            <span></span>
          </vaadin-tab>
      
          <vaadin-tab id="3" @click="${this.changeIronPage}">
            <iron-icon id="p3" icon="vaadin:cog"></iron-icon>
            <span></span>      
          </vaadin-tab>
        
          <vaadin-tab id="4" @click="${this.changeIronPage}">
            <iron-icon id="p4" icon="vaadin:envelope"></iron-icon>
            <span></span>
          </vaadin-tab>      
    
        </vaadin-tabs>  
      </div>       
    `;
  }

  // Only on first load
  firstUpdated() {
    // Disables the checkbox for auto-generating message on first load. Both date
    // and time needs to be selected for it to be enabled
    this.shadowRoot.getElementById('checkbox').disabled = true;

    // Disables both 'send' and 'delete' button on first load. There must be
    // some content in the 'text-area' for 'send' button to be enabled, and either
    // content in date, time and/or 'text-are' for 'delete' button to be enabled.
    // The buttons will optionally be enabled inside 'updated()'
    this.shadowRoot.getElementById('send-button').disabled = true;
    this.shadowRoot.getElementById('delete-button').disabled = true;

    // Google charts:
    const chart = this.shadowRoot.getElementById('chart');
    GoogleCharts.load(drawChart, {'packages':['corechart'], 'language': 'no'});
    
    function drawChart() {
  
      // Standard google charts functionality is available as GoogleCharts.api after load
      const data = new google.visualization.DataTable();
      data.addColumn('string', 'Topping');
      data.addColumn('number', 'Slices');
      data.addRows([
        ['Mushrooms', 3],
        ['Onions', 1],
        ['Olives', 1], 
        ['Zucchini', 1],
        ['Pepperoni', 2]
      ]);

      const options = {'title':'How Much Pizza I Ate Last Night',
      'width':400,
      'height':300};
      
      const pie_1_chart = new GoogleCharts.api.visualization.LineChart(chart);
      pie_1_chart.draw(data, options);
    }    

    // Parse the formatted birthdate and 5-digit social num into an 11-digit social num
    var birthdateParts = this.athlete.birthdate.split('/'); // An array containing day, month and year
    var day = birthdateParts[0];
    if (day.length === 1) day = '0' + day;
    var month = birthdateParts[1];
    if (month.length === 1) month = '0' + month;
    var year = birthdateParts[2].slice(2);
    this._fullSocialNum = day + month + year + ' ' + this.athlete.social_num;

    // '@change' is not available yet in vaadin-time-picker.
    // (see '@change' in vaadin-date-picker) 
    const timePicker = this.shadowRoot.getElementById('time-picker');
    timePicker.addEventListener('value-changed', e => {
      const value = timePicker.value;
      this.time = value;
    }); 

    // Updates 'this.todayDate'. Used to set 'min' property of 'date-picker. 
    // Can't choose any dates before 'this.today', which is set in 'this.dateToday()''
    this.dateToday();

    // Customizing vaadin-date-picker to norwegian:
    const datePicker = this.shadowRoot.querySelector('vaadin-date-picker#date-picker');

    datePicker.i18n = {
      week: 'uke',
      calendar: 'kalender',
      clear: 'tøm',
      today: 'i dag',
      cancel: 'avbryt',
      firstDayOfWeek: 1,
      monthNames:
      'januar_febraur_mars_april_mai_juni_juli_august_september_oktober_november_desember'.split('_'),
      weekdays: 'søndag_mandag_tirsdag_onsdag_torsdag_fredag_lørdag_søndag'.split('_'),
      weekdaysShort: 'sø_ma_ti_on_to_fr_lø'.split('_'),
      // Formats picked date to norwegian:
      //'date' is an object -> { day: ..., month: ..., year: ... }
      // need to format object to string in desired format. Here: DD.MM.YYY format: 
      // Note: The argument month is 0-based. This means that January = 0 and December = 11.
      formatDate: function(date) {      
        // to string
        date = JSON.stringify(date);  
        // to array
        let dateSplit = date.split(','); 
        // seperate day, month and year from array
        let day = dateSplit[0];           
        let month = dateSplit[1];
        let year = dateSplit[2];
        // delete everything except digits
        day = day.replace(/[\D]+/g, '');  
        month = month.replace(/[\D]+/g, '');
        year = year.replace(/[\D]+/g, '');
         // increase value of 'month' by 1. See Note.
        month = (+month+1)+'';
        // add '0' to day and month if one digit
        if (day.length == 1) day = '0' + day;
        if (month.length == 1) month = '0' + month;       
        // adding '/' to string -> DD.MM.YYYY
        let fullDateFormat = day + '.' + month + '.' + year;
        return fullDateFormat;
      },
      // Calendar title, i.e mai 2019
      formatTitle: function(monthName, fullYear) {
        return monthName + ' ' + fullYear;
      }
    };
    // 'firstDatOfWeek' NEEDS to my monday for week numbers to show
    datePicker.set('showWeekNumbers', true); 
    
  } // 'firstUpdated()' end

    
  // Runs every time a property is changed, i.e 'this.message'
  updated() {    
    console.log('updated()'); 

    // Checkbox is disabled on first load. Checks if BOTH time and date is selected
    if (this.date && this.time) this.shadowRoot.getElementById('checkbox').disabled = false;
    else this.shadowRoot.getElementById('checkbox').disabled = true;

    // Checks if message, date and/or time is set to enable/disable 'delete-button'
    if (this.message || this.date || this.time) this.shadowRoot.getElementById('delete-button').disabled = false;
    else this.shadowRoot.getElementById('delete-button').disabled = true;

    // Checks if there is a message, if so, enables the send button
    if (this.message) this.shadowRoot.getElementById('send-button').disabled = false;
    else this.shadowRoot.getElementById('send-button').disabled = true;
    
    //
    const dialog = this.shadowRoot.querySelector('vaadin-dialog#dialog');
    dialog.renderer = (root, dialog) => {
      // Check if there is a DOM generated with the previous renderer call to update its content instead of recreation
      if (root.firstElementChild) return;

      const div = window.document.createElement('div');
      const okButton = window.document.createElement('vaadin-button');
      okButton.setAttribute('theme', 'primary');
      okButton.textContent = 'OK';
      okButton.setAttribute('style', 'margin-right: 1em');      

      if (this.message) {
        div.textContent = 'Sende melding?';
        
        const br = window.document.createElement('br');

        okButton.addEventListener('click', () => {
          this.sendMessage();
          dialog.opened = false;
        });      

        const cancelButton = window.document.createElement('vaadin-button');
        cancelButton.textContent = 'Avbryt';
        cancelButton.addEventListener('click', () => {
          dialog.opened = false;
        });

        root.appendChild(div);
        root.appendChild(br);
        root.appendChild(okButton);
        root.appendChild(cancelButton);
      }
      else {
        div.textContent = 'Ingen melding...';

        okButton.addEventListener('click', () => {
          dialog.opened = false;
          return;
        });      

        root.appendChild(div);
        root.appendChild(okButton);
      }
    }

    this.shadowRoot.getElementById('send-button').addEventListener('click', e => {
      dialog.opened = true;
    });  
  
  } // 'updated()' end

  //
  dateToday() {
    let today = new Date();
    let dd = today.getDate();
    let mm = today.getMonth() + 1; // January is 0!
    let yyyy = today.getFullYear();

    if (dd < 10) dd = '0' + dd;
    if (mm < 10) mm = '0' + mm;
    // The format required for 'date-picker' min property   
    this.todayDate = yyyy + '-' + mm + '-' + dd;  
  }

  notification() {
    const notification = this.shadowRoot.querySelector('vaadin-notification#notification');
    notification.open();
       
    notification.renderer = function(root) {
      // Check if there is a content generated with the previous renderer call not to recreate it.
      if (root.firstElementChild) {
        return;
      }

    const container = window.document.createElement('div');
    const boldText = window.document.createElement('b');
    boldText.textContent = 'Melding sendt';

    container.appendChild(boldText);
    root.appendChild(container);
    };
  }
  
  datePicked() {
    this.date = this.shadowRoot.getElementById('date-picker').value; 
    // The '.value' is in YYYY-MM-DD format.
    // Formatting it to DD.MM.YYYY:
    // To array
    this.date = this.date.split('-');
    // Seperate day, month and year from array
    let day = this.date[2];
    let month = this.date[1];
    let year = this.date[0];
    // Constructing new date format
    this.date = day + '.' + month + '.' + year;
  }

  dateValue(e) {
    const dateElement = this.shadowRoot.getElementById('date-picker');
    if (!dateElement.value) this.date = '';
  }
  
  updateMessage(e) {
    this.message = e.target.value;
  }

  deleteData() {
    // delete contents and set 'checbox' to disabled and unchecked
    const textArea = this.shadowRoot.getElementById('text-area');
    const checkBox = this.shadowRoot.getElementById('checkbox');

    this.message = '';  
    this.messageTemp = '';
    this.date = '';
    checkBox.checked = false;
    checkBox.disabled = true;
    textArea.value = '';
    this.shadowRoot.getElementById('date-picker').value = '';
    this.shadowRoot.getElementById('time-picker').value = '';
  }


  autoMessageGen() {
    let textArea = this.shadowRoot.getElementById('text-area');
    let checkbox = this.shadowRoot.getElementById('checkbox');

    if (checkbox.checked) {
      if (this.message) {
        this.messageTemp = this.message;
        textArea.value += '\n\n' + this.autoMessageText;
        this.message += '\n\n' + this.autoMessageText;  
      }
      else {
        textArea.value += this.autoMessageText;
        this.message += this.autoMessageText; 
      }
    }    
    else  {
      if (this.messageTemp) {
        textArea.value = this.messageTemp;
        this.message = this.messageTemp;
      }
      else {
        textArea.value = '';
        this.message = '';
      }
    } 
  }

  sendMessage() {
    // Check if there is a message written inside '#text-area' by checking 'this.message'
    if(this.message) { 
      this.notification();
      console.log('Melding sendt:\n\n' + this.message);
      // Delete contents
      this.deleteData();
    }
    else console.log('Ingen melding');
  }



  changeIronPage(e) {
    this.ironPage =  e.currentTarget.id;
  }

  back() {
    this.dispatchEvent(new CustomEvent('backButtonClicked'));
  }

}

window.customElements.define('athlete-view', ViewAthlete);
