import { html, css } from 'lit-element';
import { PageViewElement } from './page-view-element.js';
import { connect } from 'pwa-helpers/connect-mixin.js';
import { store } from '../store.js';
import { SharedStyles } from './shared-styles.js';

import { formDataToJson } from '../helpers/form-data.js';
import { appurl } from '../helpers/url.js';
import { fetchResource } from '../actions/server-fetch.js';
import { addAthlete, addInjury, addMessage, readMessagesInAthlete, readMessagesInInjury, newIncomingMessage } from '../actions/inbox.js';
import session from '../reducers/session.js';

import './inbox-content.js';

import inbox from '../reducers/inbox.js';
store.addReducers({
  inbox
});

class InboxOverview extends connect(store)(PageViewElement) {
  static get properties(){
    return {
        _user: { type: Number },
        _userType: { type: Number },
        _currentAthleteId: { type: Number },
        _currentInjuryId: { type: String },
	      _currentInboxId: {type: String },
        _athletes: { type: Object },
        _injuries: { type: Object },
        _messages: { type: Object },
        _unreadInjuries: { type: Object },
        _unreadAthletes: { type: Object }
    };
  }

  constructor(){
    super();
    this._currentInjuryId = "i0";
    this._currentInboxId = "i0";
    this._currentAthleteId = 0;
  }

  static get styles() {
    return [
      SharedStyles,
      css`
        .page-wide-content-container {
          display: flex;
          flex-direction: row;
          justify-content: flex-start;
          min-height: calc(100vh - 210px);
          min-height: -moz-calc(100vh - 210px);
          min-height: -webkit-calc(100vh - 210px);
        }

        .inbox-info {
          flex-basis: 25%;
          margin-right: 10px;
          border-right: 1px solid #eee;
          max-height: calc(100vh - 210px);
          max-height: -moz-calc(100vh - 210px);
          max-height: -webkit-calc(100vh - 210px);
          overflow-y: scroll;
        }
        .inbox-content {
          flex-basis: 75%;
          padding: 14px;
          display: flex;
          flex-direction: column;

        }
        .user-container {
          display: flex;
          flex-direction: column;
          border-bottom: none;
        }
        .user-info, .injury {
          cursor: pointer;
        }
        .injury {
          padding: 5px 0 0 5px;
          margin-bottom: 5px;
        }
        .inbox {
          min-height: calc(100vh - 210px);
          min-height: -moz-calc(100vh - 210px);
          min-height: -webkit-calc(100vh - 210px);
          display: flex;
          flex-direction: column;
          justify-content: space-between;
        }
        .email {
          color: var(--dark-sand-color);
        }

      `
    ];
  }

  render() {
    return html`
    <div class="page-wide-content-container">

      <div class="inbox-info">
        ${!this._isObjectEmpty(this._athletes) ?
          (Object.keys(this._athletes).map((key) => {
            const mainInboxId = this._setMainInboxId(this._athletes[key].user_id);
            const mainInboxButton = mainInboxId.concat('button');
            const fontWeight = (typeof this._unreadAthletes[key] !== 'undefined') ?
                               'font-weight:' + this._unreadAthletes[key] + ';' :
                               'font-weight:' + 'normal' + ';';
            return html`
            <div class="user-container">
              <div class="user-info" @click="${this._displayChosenAthletesInjuries}" data-index="${this._athletes[key].user_id}">
                            <div id=${mainInboxButton} style="${fontWeight}">
                              ${this._user == this._athletes[key].user_id ? html`Hovedtråd`: html`${this._athletes[key].first_name} ${this._athletes[key].last_name}`}
                            </div>
                            <div class="email">
                              ${this._user == this._athletes[key].user_id ? html``: html`${this._athletes[key].email}`}
                            </div>
              </div>
              <div class="injuries" style="display:none;" id="${this._athletes[key].user_id}">
                Skader:
                ${Object.keys(this._athletes[key].injuries).map((iKey) => {
                  if(iKey !== '0'){
                    const inboxId = this._setInjuryId(this._injuries[iKey].injury_id);
                    const injuryId = inboxId.concat('button');
                    const fontWeight = (typeof this._unreadInjuries[iKey] !== 'undefined') ?
                                        'font-weight:' + this._unreadInjuries[iKey] + ';' :
                                        'font-weight:' + 'normal' + ';';
                    return html`
                      <div class="injury" id=${injuryId} @click="${this._displayChosenMessage}" data-index="${inboxId}" style="${fontWeight}">
                        ${this._injuries[iKey].title}
                      </div>
                    `;
                  }
                })}
              </div>
            </div>
          `})) :
          (Object.keys(this._injuries).map((key) => {
            const inboxId = this._setInjuryId(this._injuries[key].injury_id);
            return html`
              <div class="injury" @click="${this._displayChosenMessage}" data-index="${inboxId}">${this._injuries[key].title}</div>
          `}))
        }
      </div>

      <div class="inbox-content" id="inbox-content">
        <div class="inbox" style="display:block;" id="i0">
          velg en meldings-tråd
        </div>
        ${Object.keys(this._athletes).map((key) => {
          const inboxId = this._setMainInboxId(this._athletes[key].user_id);
          const messages = {};
          let lastMessage;
          Object.keys(this._athletes[key].messages).map((mKey) => {
            messages[mKey] = this._messages[mKey];
          });
          return html`
          <inbox-content style="display:none;" id="${inboxId}" .user="${this._user}"
                        .currentAthleteId="${this._athletes[key].user_id}"
                        .currentInjuryId="0"
			.messages="${messages}"></inbox-content>

        `})}
        ${Object.keys(this._injuries).map((key) => {
          const inboxId = this._setInjuryId(this._injuries[key].injury_id);
          const messages = {};
          Object.keys(this._injuries[key].messages).map((mKey) => {
            messages[mKey] = this._messages[mKey];
          });
          return html`
          <inbox-content style="display:none;" id="${inboxId}" .user="${this._user}"
                        .currentInjuryId="${this._injuries[key].injury_id}"
                        .currentAthleteId="0"
			.messages="${messages}"></inbox-content>

        `})}
      </div>

    </div>
    `
  }

  _setInjuryId(id){
    const iId = 'i'+id;
    return iId;
  }

  _setMainInboxId(id){
    const iId = 'a'+id;
    return iId;
  }

  _isObjectEmpty(Obj) {
    for(var key in Obj) {
      if(Obj.hasOwnProperty(key)){
        return false;
      }
    }
    return true;
  }

  _displayChosenMessage(e){
    if(this._unreadInjuries[Number(e.currentTarget.dataset['index'].slice(1))]){
      const athleteInboxId = Object.keys(this._athletes).filter(key => (Object.keys(this._athletes[key].injuries).filter(i => i == Number(e.currentTarget.dataset['index'].slice(1)))).length > 0);
      this._readMessage(e.currentTarget.dataset['index'].slice(1), "injury");
      store.dispatch(readMessagesInInjury(Number(e.currentTarget.dataset['index'].slice(1))));
    }
    this.shadowRoot.getElementById(this._currentInboxId).style.display = "none";
    if(e.currentTarget.dataset['index'].startsWith("i")){
      this._currentInboxId = e.currentTarget.dataset['index'];
    } else {
      this._currentInboxId = 'a' + e.currentTarget.dataset['index'];
    }
    this.shadowRoot.getElementById(this._currentInboxId).style.display = "block";
  }

  _displayChosenAthletesInjuries(e){
    this._readMessage(e.currentTarget.dataset['index'].slice(1), "athlete");
    if(this._unreadAthletes[e.currentTarget.dataset['index']]){
      this._readMessage(e.currentTarget.dataset['index'], "athlete");
      store.dispatch(readMessagesInAthlete(e.currentTarget.dataset['index']));
    }
    if(this._currentAthleteId !== 0) {
      if(this._currentAthleteId !== Number(e.currentTarget.dataset['index'])){
        this.shadowRoot.getElementById(this._currentAthleteId.toString()).style.display = "none";
      }
      this.shadowRoot.getElementById(this._currentInboxId).style.display = "none";
      this._currentInboxId = 'a' + e.currentTarget.dataset['index'];
      this.shadowRoot.getElementById(this._currentInboxId).style.display = "block";
    }
    if(this._currentAthleteId == Number(e.currentTarget.dataset['index'])){
      //this.shadowRoot.getElementById(this._currentAthleteId.toString()).style.display = "none";
      //this._currentAthleteId = 0;
      this.shadowRoot.getElementById(this._currentInboxId).style.display = "none";
      this._currentInboxId = 'a' + this._currentAthleteId;
      this.shadowRoot.getElementById(this._currentInboxId).style.display = "block";
    } else {
      this._currentAthleteId = Number(e.currentTarget.dataset['index']);
      this.shadowRoot.getElementById(this._currentAthleteId.toString()).style.display = "block";
      // display chosen athletes main inbox thread
      this.shadowRoot.getElementById(this._currentInboxId).style.display = "none";
      this._currentInboxId = 'a' + e.currentTarget.dataset['index'];
      this.shadowRoot.getElementById(this._currentInboxId).style.display = "block";
    }
  }

  async firstUpdated(){
    const url = appurl + "/api/inbox/" + this._user;
    const fetchOptions = {
      method: "GET"
    };
    store.dispatch(fetchResource(url, fetchOptions)).then(() => {
      const data = store.getState().resources.data;
      if(data.status == 'OK'){

        Object.keys(data.athletes).map((key) => {
          store.dispatch(addAthlete(data.athletes[key]));
        });
        Object.keys(data.injuries).map((key) => {
          store.dispatch(addInjury(data.injuries[key]));
        });
        Object.keys(data.messages).map((key) => {
          store.dispatch(addMessage(data.messages[key]));
        });
        Object.keys(data.newMessages).map((key) => {
          store.dispatch(newIncomingMessage(key));
        });

      } else { // TODO handle error
        console.log('error');
      }
    })
  }

  stateChanged(state){
    if(this._messages !== state.inbox.messages){
      this._messages = state.inbox.messages;
    }
    if(this._user !== state.session.userID){
      this._user = state.session.userID;
    }
    if(this._userType !== state.session.userType){
      this._userType = state.session.userType;
    }
    this._athletes = state.inbox.athletes;
    this._injuries = state.inbox.injuries;

    if(this._unreadAthletes !== state.inbox.unreadAthletes){
      this._unreadAthletes = state.inbox.unreadAthletes;
    }
    if(this._unreadInjuries !== state.inbox.unreadInjuries){
      this._unreadInjuries = state.inbox.unreadInjuries;
    }
  }

  async _readMessage(inbox, type) {
    //const formElement = this.shadowRoot.getElementById("injuryEdit");
    const jsonFormData = `{"id":"${this._user}","inbox":"${inbox}","type":"${type}"}`;

    const url = appurl + "/api/inbox";
    const fetchOptions = {
      method: "PUT",
      headers: { "Content-Type": "application/json" },
      body: jsonFormData
    };
    store.dispatch(fetchResource(url, fetchOptions));
  }
}

window.customElements.define('inbox-overview', InboxOverview);
