import { html, css } from 'lit-element';
import { PageViewElement } from './page-view-element.js';
import { connect } from 'pwa-helpers/connect-mixin.js';
import { store } from '../store.js';
import { SharedStyles } from './shared-styles.js';

import { fixDrawerPosition } from '../helpers/position-fix.js';
import { formDataToJson } from '../helpers/form-data.js';
import { appurl } from '../helpers/url.js';

import { fetchResource } from '../actions/server-fetch.js';
import { addInjury } from '../actions/injuries.js';

import injuries from '../reducers/injuries.js';
store.addReducers({
  injuries
});

/*
  Simple injury reporting form for athletes.
*/
class NewInjury extends connect(store)(PageViewElement) {
  static get properties(){
    return {
      _drawerOpened: { type: Boolean },
      _loggedIn: { type: Boolean },
      _userType: { type: Number },
      _userID: { type: Number }
    };
  }

  static get styles() {
    return [
      SharedStyles,
      css`
        .page-content-container {
          margin-top: 20px;
        }
        input {
          color: #333;
        }
      `
    ];
  }

  stateChanged(state) {
    this._drawerOpened = state.app.drawerOpened;
    this._loggedIn = state.session.loggedIn;
    this._userType = state.session.userType;
    this._userID = state.session.userID;
  }

  render() {
    fixDrawerPosition(this);
    return html`
      <div id="page" class="page-content-container">
        ${this._loggedIn && this._userType === 1 ? html `
        <form id="regInjury" @submit="${this._submitForm}">
          <input type="hidden" name="athleteId" value="${this._userID}">
          <div class="form-row">
            <div class="form-input">
            <label class="label">Hvor er du skadet?</label>
              <input type="text" name="title" placeholder="Hvor er du skadet?">
            </div>
          </div>
          <div class="form-row">
            <div class="form-input">
            <label class="label">Valgfri melding til helsepersonell</label>
              <textarea form="regInjury" rows="4" name="message" placeholder="Valgfri melding til helsepersonell"></textarea>
            </div>
          </div>
          <div class="form-row">
            <label class="checkbox-box">Jeg vil at helsepersonell skal kontakte meg for å avtale time angående denne skaden
              <input type="checkbox" name="contact" checked="checked">
              <span class="checkmark"></span>
            </label>
          </div>
          <div class="form-row">
            <input class="input-app-button" type="submit" value="Send inn">
          </div>
        </form>
        ` : html`<p>Du er ikke logget inn, eller har ikke tilgang til denne siden.</p>`}
      </div>
    `
  }


  _submitForm(event) {
    const formElement = this.shadowRoot.getElementById("regInjury");
    const jsonFormData = formDataToJson(formElement);
    const url = appurl + "/api/injuries";
    const fetchOptions = {
      method: "POST",
      headers: { "Content-Type": "application/json" },
      body: jsonFormData
    };
    store.dispatch(fetchResource(url, fetchOptions)).then(() => {
      const data = store.getState().resources.data;
      console.log(data);
      if (data.status === 'OK') {
        // Immediately add the injury to state and redirect to MyInjuries,
        // so that the new injury is visible at the top of the list
        store.dispatch(addInjury(data.injury));
        this._sendNewInjuryMessage(data.injury.id, data.message);
        window.location.replace("/mineskader");
      }
    })
    event.preventDefault();
  }

  _sendNewInjuryMessage(injury, message) {
    console.log('kjører send message');
    //const formElement = this.shadowRoot.getElementById("injuryEdit");
    const jsonFormData = `{"from":"${this._userID}","injury":"${injury}","message":"${message}","type":"newInjury"}`;
    const url = appurl + "/api/inbox";
    const fetchOptions = {
      method: "POST",
      headers: { "Content-Type": "application/json" },
      body: jsonFormData
    };
    store.dispatch(fetchResource(url, fetchOptions)).then(() => {
      const data = store.getState().resources.data;
      console.log(data);

    });
  }
}



window.customElements.define('new-injury', NewInjury);
