/*
  Fix for pages containing CSS positioned elements (position != static). Hides them behind the drawer while it's open.
  Call this method in render(). Requirements:
  - All positioned elements must be encompassed by an element with id="page" (typically a div class="page-content-container")
  - The page component must store a property _drawerOpened, listen to changes in the store, and keep it updated
  Called with "this" as parameter. See new-user.js for an example.
*/
export function fixDrawerPosition(obj) {
  const page = obj.shadowRoot.getElementById('page');
  if (obj._drawerOpened) {
    page.style.zIndex = '-1'; // Hide page behind drawer
  } else {
    setTimeout(() => { // Wait for drawer closing animation before showing page again
      try {
        page.style.zIndex = 'auto';
      } catch (err) {} // Suppress error when the page element is not found on first load
    }, 150);
  }
}

/*
  Here are some older notes for this bug:

  NOTE:
  Page content must be styled without the position attribute. Any elements with
  position: relative / absolute / fixed / sticky will be displayed IN FRONT OF
  main elements such as header and drawer. This means we can't stack elements
  on top of each other.

  NOTE 2:
  Found a workaround.
  Each page that contains positioned elements listens for _drawerOpened in the store.
  When the drawer is opened, set the page's root element (e.g. a .page-content-container)
  to z-index = -1. This forces it behind the drawer. When closed, set to auto to make the
  content available again.
  Implemented in new-user.js, see imports and render().
*/
