// Copy the entire state to localStorage
export const saveState = (state) => {
  let stringifiedState = JSON.stringify(state);
  localStorage.setItem("app_state", stringifiedState);
}

// Read localStorage, but only copy session into state
export const loadState = () => {
  let json = localStorage.getItem("app_state") || '{}';
  let state = JSON.parse(json);
  if (state) {  // There is data to load

    // Unset the parts of state we don't want to load
    state.app = {
      page: '',
      offline: false,
      drawerOpened: false,
      snackbarOpened: false,
    };
    state.inbox = {
      newMessages: 0,
      unreadInjuries: {},
      unreadAthletes: {},
      athletes: {},
      injuries: {},
      messages: {}
    };
    state.injuries = {
      injuries_active: [],
      injuries_archived: []
    };
    state.resources = {
      data: {},
      loading: false,
      error: null
    };

    if (state.session) { // Load session information
      if (Date.now() > state.session.expires) { // Session has expired
        state.session.loggedIn = false;
        state.session.userID = null;
        state.session.userType = null;
        state.session.expires = null;
      }
    }
    return state;
  } else {
    return undefined;  // To use the defaults in the reducers
  }
}
